package hu.uw.pallergabor.blelight;

import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothGatt;
import android.bluetooth.BluetoothGattCallback;
import android.bluetooth.BluetoothGattCharacteristic;
import android.bluetooth.BluetoothGattDescriptor;
import android.bluetooth.BluetoothGattService;
import android.bluetooth.BluetoothManager;
import android.bluetooth.BluetoothProfile;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.widget.TextView;
import android.widget.Toast;

import java.util.UUID;

/**
 * Created by paller on 2014.12.26..
 */
public class BLELightActivity extends Activity {
    public static String EXTRAS_DEVICE_NAME = "device_name";
    public static String EXTRAS_DEVICE_ADDRESS = "device_address";
    private Handler mHandler;
    private String deviceName;
    private String deviceAddress;
    private BluetoothAdapter bluetoothAdapter;
    private static final String LOG_TAG = "BL.BLActivity";

    public static final UUID BLELIGHT_SERVICE_UUID = UUID.fromString("274b15b0-b9cd-4e5e-94c4-1248b42b82f8");
    public static final UUID BLELIGHT_LIGHT_ID_CHARACTERISTIC_UUID =
            UUID.fromString("274b0100-b9cd-4e5e-94c4-1248b42b82f8");
    // 0x2902 org.bluetooth.descriptor.gatt.client_characteristic_configuration.xml
    public final static UUID CLIENT_CHARACTERISTIC_CONFIGURATION_UUID = UUID.fromString("00002902-0000-1000-8000-00805F9B34FB");
    private TextView lightIDTV;
    private TextView temperatureTV;
    private TextView humidityTV;
    private boolean isNotificationSet;
    private boolean isConnectStarted;
    private BluetoothGattCharacteristic lightIDCharacteristic;
    private BluetoothGattDescriptor lightIDConfigDescriptor;

    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_blelight);
        final Intent intent = getIntent();
        deviceName = intent.getStringExtra(EXTRAS_DEVICE_NAME);
        deviceAddress = intent.getStringExtra(EXTRAS_DEVICE_ADDRESS);

        getActionBar().setTitle(deviceName);
        mHandler = new Handler();
        final BluetoothManager bm =
                (BluetoothManager) getSystemService(Context.BLUETOOTH_SERVICE);
        bluetoothAdapter = bm.getAdapter();
        lightIDTV = (TextView)findViewById(R.id.blelight_light_id_tv);
        temperatureTV = (TextView)findViewById(R.id.blelight_temperature_tv);
        humidityTV = (TextView)findViewById(R.id.blelight_humidity_tv);
        isNotificationSet = false;
        isConnectStarted = false;
    }

    @Override
    protected void onResume() {
        super.onResume();
        BluetoothDevice device = bluetoothAdapter.getRemoteDevice(deviceAddress);
        if( device != null && !isConnectStarted ) {
            isConnectStarted = true;
            device.connectGatt(this,false,gattCallback);
            Log.d(LOG_TAG, "connectGatt");
        } else
            Log.d( LOG_TAG, "device==null");
    }

    public void onBackPressed () {
        if( btGatt == null ) {        // Finish normally if we don't have any connection
            super.onBackPressed();
        } else {                    // Else we start the disconnection and finish the activity only if the disconnection finished
            Log.d( LOG_TAG, "btGatt.disconnect()");
            btGatt.disconnect();
        }
    }

    private BluetoothGatt btGatt = null;
    private BluetoothGattService gattService;

    private void updateLightID(byte ctdata[]) {
        String data = null;
        float tf;

        if( ctdata.length >= 3 ) {
            int lightData = (((int)ctdata[0]) & 0xFF ) |
                    ((((int)ctdata[1]) << 8 ) & 0xFF00) |
                    ((((int)ctdata[2]) << 16 ) & 0xFF0000);
            int selector = ( lightData & 0xFF0000 ) >> 16;
            LightDataUpdateTask ldut = null;
            switch( selector ) {
                case 0:
                    ldut = new LightDataUpdateTask( getString(R.string.blelight_light_id_init),
                            getString(R.string.blelight_temperature_init),
                            getString(R.string.blelight_humidity_init));
                    break;

                case 1:
                    data = "Station ID: 0x"+Integer.toHexString( lightData & 0xFFFF );
                    ldut = new LightDataUpdateTask( data,null,null);
                    break;

                case 2:
                    tf = (float)( ( lightData & 0xFFFF ) - 10000 );
                    tf = tf / 100.0f;
                    data = "Temperature: "+Float.toString( tf )+" °C";
                    ldut = new LightDataUpdateTask( null,data,null);
                    break;

                case 3:
                    tf = (float)(lightData & 0xFFFF );
                    tf = tf / 100.0f;
                    data = "Humidity: "+Float.toString( tf )+"%";
                    ldut = new LightDataUpdateTask( null,null,data);
                    break;
            }
            if( ldut != null )
                mHandler.post(ldut);
        }
    }

    class LightDataUpdateTask implements Runnable {
        public LightDataUpdateTask( String lightID, String temperature, String humidity ) {
            this.lightID = lightID;
            this.temperature = temperature;
            this.humidity = humidity;
        }

        public void run() {
            if( lightID != null )
                lightIDTV.setText( lightID);
            if( temperature != null )
                temperatureTV.setText( temperature);
            if( humidity != null)
                humidityTV.setText( humidity );
        }

        String lightID;
        String temperature;
        String humidity;
    }

    private BluetoothGattCallback gattCallback = new BluetoothGattCallback() {
        public void onConnectionStateChange(BluetoothGatt gatt, int status, int newState) {

            if( newState == BluetoothProfile.STATE_DISCONNECTED) {
                Log.d(LOG_TAG, "STATE_DISCONNECTED");
                mHandler.postDelayed(new Runnable() {
                    @Override
                    public void run() {
                        btGatt.close();
                        btGatt = null;
                        isConnectStarted = false;
// The back button's default handling is disabled if we have a connection. By pressing back.
// the disconnection starts but we finish the activity if the disconnection is completed.
                        Log.d(LOG_TAG, "STATE_DISCONNECTED2");
                        mHandler.postDelayed(new Runnable() {
                            public void run() {
                                finish();
                            }
                        },500L);
                    }
                }, 500L);

            } else
            if( newState == BluetoothProfile.STATE_CONNECTED) {
                Log.d( LOG_TAG, "STATE_CONNECTED");
                btGatt = gatt;
                boolean success = gatt.discoverServices();
                Log.d( LOG_TAG, "discoverServices: "+success);
            }
        }

        // New services discovered
        public void onServicesDiscovered(BluetoothGatt gatt, int status) {
            if (status == BluetoothGatt.GATT_SUCCESS) {
                Log.d( LOG_TAG, "services discovered");
                gattService = gatt.getService(BLELIGHT_SERVICE_UUID);
                if( gattService == null ) {
                    mHandler.post(new Runnable(){
                        public void run() {
                            Toast.makeText(BLELightActivity.this, R.string.blelight_not_supported, Toast.LENGTH_LONG).show();
                        }
                    });
                } else {
                    lightIDCharacteristic = gattService.getCharacteristic(BLELIGHT_LIGHT_ID_CHARACTERISTIC_UUID);
                    int props = lightIDCharacteristic.getProperties();
                    Log.d( LOG_TAG, "props: "+props);
                    if( ( props & BluetoothGattCharacteristic.PROPERTY_INDICATE ) != 0)
                        Log.d( LOG_TAG,"PROPERTY_INDICATE");
                    if( ( props & BluetoothGattCharacteristic.PROPERTY_NOTIFY ) != 0)
                        Log.d( LOG_TAG,"PROPERTY_NOTIFY");
                    if( ( props & BluetoothGattCharacteristic.PROPERTY_READ ) != 0)
                        Log.d( LOG_TAG, "PROPERTY_READ");
                    if( ( props & BluetoothGattCharacteristic.PROPERTY_WRITE ) != 0)
                        Log.d( LOG_TAG, "PROPERTY_WRITE");
                    if( ( props & BluetoothGattCharacteristic.PROPERTY_SIGNED_WRITE ) != 0)
                        Log.d( LOG_TAG, "PROPERTY_SIGNED_WRITE");
                    if( ( props & BluetoothGattCharacteristic.PROPERTY_BROADCAST ) != 0)
                        Log.d( LOG_TAG, "PROPERTY_BROADCAST");
                    boolean success = gatt.readCharacteristic(lightIDCharacteristic);
                    Log.d( LOG_TAG,"read operation status: "+success);
                }
            } else {
                Log.w(LOG_TAG, "onServicesDiscovered received: " + status);
            }
        }

        public void onCharacteristicWrite (
                BluetoothGatt gatt,
                BluetoothGattCharacteristic characteristic,
                int status) {
            String success = status == BluetoothGatt.GATT_SUCCESS ?
                    "Success" :
                    "Failure";
            String message =
                    "Write operation on "+
                            characteristic.getUuid().toString()+
                            " : "+success;
            Log.d( LOG_TAG, "status: "+status+" : "+message);
        }

        public void onCharacteristicRead (BluetoothGatt gatt, BluetoothGattCharacteristic characteristic, int status) {
            String message =
                    "Characteristic read: "+
                            characteristic.getUuid().toString();
            Log.d( LOG_TAG, message);
            byte v[] = characteristic.getValue();
            for( int i = 0 ; i < v.length ; ++i ) {
                Log.d( LOG_TAG, "v["+i+"]: "+v[i]);
            }
            updateLightID(v);

            if( characteristic.equals( lightIDCharacteristic ) && !isNotificationSet ) {
                Log.d( LOG_TAG, "setting notification");
                gatt.setCharacteristicNotification(lightIDCharacteristic, true);
                lightIDConfigDescriptor = lightIDCharacteristic.getDescriptor(CLIENT_CHARACTERISTIC_CONFIGURATION_UUID);
                if (lightIDConfigDescriptor != null) {
                    lightIDConfigDescriptor.setValue(BluetoothGattDescriptor.ENABLE_NOTIFICATION_VALUE);
                    gatt.writeDescriptor(lightIDConfigDescriptor);
                } else {
                    Log.e(LOG_TAG, "Receive Characteristic can not be configured.");
                }
                isNotificationSet = true;
            }
        }

        public void onCharacteristicChanged (
                BluetoothGatt gatt,
                BluetoothGattCharacteristic characteristic) {
            String message =
                    "Characteristic changed: "+
                            characteristic.getUuid().toString();
            Log.d( LOG_TAG, message);
            byte v[] = characteristic.getValue();
            for( int i = 0 ; i < v.length ; ++i ) {
                Log.d( LOG_TAG, "v["+i+"]: "+v[i]);
            }
            updateLightID(v);
        }
    };

}
