package hu.uw.pallergabor.blelight;

import hu.uw.pallergabor.ble.adparser.AdElement;
import hu.uw.pallergabor.ble.adparser.AdParser;
import hu.uw.pallergabor.ble.adparser.Type128BitUUIDs;
import hu.uw.pallergabor.ble.adparser.Type16BitUUIDs;

import java.util.ArrayList;
import java.util.UUID;

import android.app.Activity;
import android.app.ListActivity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothGattService;
import android.bluetooth.BluetoothManager;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;


public class MainActivity extends ListActivity {
	private static final String LOG_TAG = "BLELight.MA";
    private LeDeviceListAdapter mLeDeviceListAdapter;
    private Handler mHandler;
    private boolean mScanning;
    private BluetoothAdapter mBluetoothAdapter;
    private BluetoothGattService gattService;
    private Intent launchIntent;
    private BluetoothDevice currentDevice;
    private static final int REQUEST_ENABLE_BT = 1;
    // Stops scanning after 10 seconds.
    private static final long SCAN_PERIOD = 10000;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        getActionBar().setTitle(R.string.title_devices);
        mHandler = new Handler();

        // Use this check to determine whether BLE is supported on the device.  Then you can
        // selectively disable BLE-related features.
        if (!getPackageManager().hasSystemFeature(PackageManager.FEATURE_BLUETOOTH_LE)) {
            Toast.makeText(this, R.string.ble_not_supported, Toast.LENGTH_SHORT).show();
            finish();
        }

        // Initializes a Bluetooth adapter.  For API level 18 and above, get a reference to
        // BluetoothAdapter through BluetoothManager.
        final BluetoothManager bluetoothManager =
                (BluetoothManager) getSystemService(Context.BLUETOOTH_SERVICE);
        mBluetoothAdapter = bluetoothManager.getAdapter();

        // Checks if Bluetooth is supported on the device.
        if (mBluetoothAdapter == null) {
            Toast.makeText(this, R.string.error_bluetooth_not_supported, Toast.LENGTH_SHORT).show();
            finish();
            return;
        }

    }


    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.main, menu);
        if (!mScanning) {
            menu.findItem(R.id.menu_stop).setVisible(false);
            menu.findItem(R.id.menu_scan).setVisible(true);
            menu.findItem(R.id.menu_refresh).setActionView(null);
        } else {
            menu.findItem(R.id.menu_stop).setVisible(true);
            menu.findItem(R.id.menu_scan).setVisible(false);
            menu.findItem(R.id.menu_refresh).setActionView(
                    R.layout.actionbar_indeterminate_progress);
        }
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
        case R.id.menu_scan:
            mLeDeviceListAdapter.clear();
            scanLeDevice(true);
            break;
        case R.id.menu_stop:
            scanLeDevice(false);
            break;
    }
    return true;
    }

    @Override
    protected void onResume() {
        super.onResume();

        // Ensures Bluetooth is enabled on the device.  If Bluetooth is not currently enabled,
        // fire an intent to display a dialog asking the user to grant permission to enable it.
        if (!mBluetoothAdapter.isEnabled()) {
            if (!mBluetoothAdapter.isEnabled()) {
                Intent enableBtIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
                startActivityForResult(enableBtIntent, REQUEST_ENABLE_BT);
            }
        }

        // Initializes list view adapter.
        mLeDeviceListAdapter = new LeDeviceListAdapter();
        setListAdapter(mLeDeviceListAdapter);
        scanLeDevice(true);
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        // User chose not to enable Bluetooth.
        if (requestCode == REQUEST_ENABLE_BT && resultCode == Activity.RESULT_CANCELED) {
            finish();
            return;
        }
        super.onActivityResult(requestCode, resultCode, data);
    }

    @Override
    protected void onPause() {
        super.onPause();
        scanLeDevice(false);
        mLeDeviceListAdapter.clear();
    }

    @Override
    protected void onListItemClick(ListView l, View v, int position, long id) {
        currentDevice = mLeDeviceListAdapter.getDevice(position);
        if (currentDevice == null) return;
        DeviceHolder deviceHolder = mLeDeviceListAdapter.mLeHolders.get(position);
        ArrayList<AdElement> ads = deviceHolder.additionalData;
        boolean found = false;
        for( int i = 0 ; i < ads.size() ; ++i ) {
            AdElement e = ads.get(i);
            if( e instanceof Type128BitUUIDs) {
                Type128BitUUIDs tu = (Type128BitUUIDs)e;
                UUID uuids[] = tu.getUUIDs();
                for( int n = 0 ; n < uuids.length ; ++n ) {
                    if( uuids[n].compareTo(BLELightActivity.BLELIGHT_SERVICE_UUID) == 0 ) {
                        found = true;
                        break;
                    }
                }
            }
        }
        if( !found ) {
            Toast.makeText(this, R.string.blelight_not_advertised, Toast.LENGTH_LONG).show();
            return;
        }
        if (mScanning) {
            mBluetoothAdapter.stopLeScan(mLeScanCallback);
            mScanning = false;
        }
        launchIntent = new Intent(MainActivity.this, BLELightActivity.class);
        launchIntent.putExtra(BLELightActivity.EXTRAS_DEVICE_NAME, currentDevice.getName());
        launchIntent.putExtra(BLELightActivity.EXTRAS_DEVICE_ADDRESS, currentDevice.getAddress());
        startActivity(launchIntent);
    }

    private void scanLeDevice(final boolean enable) {
        if (enable) {
            // Stops scanning after a pre-defined scan period.
            mHandler.postDelayed(new Runnable() {
                @Override
                public void run() {
                    mScanning = false;
                    mBluetoothAdapter.stopLeScan(mLeScanCallback);
                    invalidateOptionsMenu();
                }
            }, SCAN_PERIOD);

            mScanning = true;
            mBluetoothAdapter.startLeScan(mLeScanCallback);
        } else {
            mScanning = false;
            mBluetoothAdapter.stopLeScan(mLeScanCallback);
        }
        invalidateOptionsMenu();
    }

    static class ViewHolder {
        TextView deviceName;
        TextView deviceAd;
        TextView deviceRssi;
        TextView deviceAddress;
    }

    class DeviceHolder {
    	BluetoothDevice device;
    	ArrayList<AdElement> additionalData;
        String additionalDataString;
    	int rssi;
    	
    	public DeviceHolder(BluetoothDevice device,
                            ArrayList<AdElement> additionalData,
                            String additionalDataString,
                            int rssi) {
    		this.device = device;
    		this.additionalData = additionalData;
            this.additionalDataString = additionalDataString;
    		this.rssi = rssi;
    	}
    }
    // Adapter for holding devices found through scanning.
    private class LeDeviceListAdapter extends BaseAdapter {
        ArrayList<BluetoothDevice> mLeDevices;
        ArrayList<DeviceHolder> mLeHolders;
        private LayoutInflater mInflator;

        public LeDeviceListAdapter() {
            super();
            mLeDevices = new ArrayList<BluetoothDevice>();
            mLeHolders = new ArrayList<DeviceHolder>();
            mInflator = MainActivity.this.getLayoutInflater();
        }

        public void addDevice(DeviceHolder deviceHolder) {
            if(!mLeDevices.contains(deviceHolder.device)) {
                mLeDevices.add(deviceHolder.device);
                mLeHolders.add(deviceHolder);
            }
        }

        public BluetoothDevice getDevice(int position) {
            return mLeDevices.get(position);
        }

        public void clear() {
            mLeDevices.clear();
            mLeHolders.clear();
        }

        @Override
        public int getCount() {
            return mLeDevices.size();
        }

        @Override
        public Object getItem(int i) {
            return mLeDevices.get(i);
        }

        @Override
        public long getItemId(int i) {
            return i;
        }

        @Override
        public View getView(int i, View view, ViewGroup viewGroup) {
            ViewHolder viewHolder;
            // General ListView optimization code.
            if (view == null) {
                view = mInflator.inflate(R.layout.listitem_device, null);
                viewHolder = new ViewHolder();
                viewHolder.deviceAddress = (TextView) view.findViewById(R.id.device_address);
                viewHolder.deviceAd = (TextView) view.findViewById(R.id.device_ad);
                viewHolder.deviceRssi = (TextView) view.findViewById(R.id.device_rssi);
                viewHolder.deviceName = (TextView) view.findViewById(R.id.device_name);
                view.setTag(viewHolder);
            } else {
                viewHolder = (ViewHolder) view.getTag();
            }

            BluetoothDevice device = mLeDevices.get(i);
            DeviceHolder deviceHolder = mLeHolders.get(i);
            final String deviceName = device.getName();
            if (deviceName != null && deviceName.length() > 0)
                viewHolder.deviceName.setText(deviceName);
            else
                viewHolder.deviceName.setText(R.string.unknown_device);
            viewHolder.deviceAddress.setText(device.getAddress());
            viewHolder.deviceAd.setText(deviceHolder.additionalDataString);
            viewHolder.deviceRssi.setText("rssi: "+Integer.toString(deviceHolder.rssi));
            return view;
        }
    }

    // Device scan callback.
    private BluetoothAdapter.LeScanCallback mLeScanCallback =
            new BluetoothAdapter.LeScanCallback() {

        @Override
        public void onLeScan(final BluetoothDevice device, int rssi, byte[] scanRecord) {
        	String deviceName = device.getName();
        	StringBuffer b = new StringBuffer();
        	int byteCtr = 0;
        	for( int i = 0 ; i < scanRecord.length ; ++i ) {
        		if( byteCtr > 0 )
        			b.append( " ");
        		b.append( Integer.toHexString( ((int)scanRecord[i]) & 0xFF));
        		++byteCtr;
        		if( byteCtr == 8 ) {
        			Log.d( LOG_TAG, new String( b ));
        			byteCtr = 0;
        			b = new StringBuffer();
        		}
        	}
        	ArrayList<AdElement> ads = AdParser.parseAdData(scanRecord);
        	StringBuffer sb = new StringBuffer();
        	for( int i = 0 ; i < ads.size() ; ++i ) {
        		AdElement e = ads.get(i);
        		if( i > 0 )
        			sb.append(" ; ");
        		sb.append(e.toString());
        	}
            boolean found = false;
            for( int i = 0 ; i < ads.size() ; ++i ) {
                AdElement e = ads.get(i);
                Log.d( LOG_TAG, "AdElement: "+e.toString());
                if( e instanceof Type128BitUUIDs) {
                    Type128BitUUIDs tu = (Type128BitUUIDs)e;
                    UUID uuids[] = tu.getUUIDs();
                    for( int n = 0 ; n < uuids.length ; ++n ) {
                        if( uuids[n].compareTo(BLELightActivity.BLELIGHT_SERVICE_UUID) == 0 ) {
                            found = true;
                            break;
                        }
                    }
                }
            }
            if( found ) {
                String additionalData = new String(sb);
                Log.d(LOG_TAG, "additionalData: " + additionalData);
                DeviceHolder deviceHolder = new DeviceHolder(device, ads, additionalData, rssi);
                runOnUiThread(new DeviceAddTask(deviceHolder));
            }
        }
    };

    class DeviceAddTask implements Runnable {
    	DeviceHolder deviceHolder;
    	
    	public DeviceAddTask( DeviceHolder deviceHolder ) {
    		this.deviceHolder = deviceHolder;
    	}
    	
    	public void run() {
            mLeDeviceListAdapter.addDevice(deviceHolder);
            mLeDeviceListAdapter.notifyDataSetChanged();    		
    	}
    }
}
